# Plugins

Like many fullstack frameworks, Perseus supports *plugins*, which allow you to extend the basic functionality of Perseus in sometimes extreme ways! However, unlike other frameworks, Perseus is already extremely customizable with usual usage, due to the way it exposes all operations directly to the user. For example, if you wanted to restructure your server, all that code is open to you directly.

In earlier version of Perseus, there was a folder called `.perseus/` that stored a large amount of internal code, and plugins were mostly used to modify that. Today, that code simply doesn't exist anymore, and everything is bundled into the main app! (With the consequence of a slightly wild `Cargo.toml`...) This means that most things aren't done with a plugin anymore.

However, for tasks that involve injecting into Perseus' build system, a plugin is usually the right tool for the job. Let's say for example that you wanted to, before building the app, identify all Sass files that were being imported in the index view and compile them to CSS. You could do this by first parsing the index view, and then compiling everything at build-time. This is doable with plugins, though the mechanics of this particular example are fairly complex.

In the general case, you can do most things in Perseus by playing around with the code that's exposed to you. If you just want to add something extra inside Perseus' internal processes though (e.g. adding a copyright header to all exported files), this is done with a plugin. If you're unsure, you can always ask in [a discussion]() or on [our Discord channel on the Sycamore server]().

Perseus' plugins are based on *actions*, which you can make your plugin use to execute arbitrary code, as per [these examples](). There are three types of actions: functional, control, and tinker. Functional actions can have many plugins connected to them (e.g. adding more templates). Control actions can have just one plugin connected to them (e.g. modifying the index view). Tinker plugins are weird, they're executed on the special command `perseus tinker`, and they were originally designed to let people modify the code inside `.perseus/` (that legacy hidden folder), but now they're just...a thing. We haven't thought of any particular use-case for them yet, but there's not really any downside in having them, and, who knows, you might one day discover that a very particular niche application requires that extra step of explicitly executing `perseus tinker` to modify stuff. As for what those tinker plugins can do: literally anything. They're given the entire filesystem and they can roam free. Heck, you could use a tinker plugin to install an application on your computer, if you really wanted to! (And that's why you should only ever use trusted plugins!)

On the note of security, plugins are extremely powerful. They can execute arbitrary code, and so can do basically whatever they want to your system. We have a list of publicly available plugins [here]() that are accompanied by little badges that indicate review by the Perseus dev team. Usually, those ones at least will be safe to use, though we strongly recommend reviewing the code of the plugins you use yourself, as we do NOT review each new version, and we do NOT keep track of changes to plugin maintainership. In other words, we take no responsibility whatsoever for anything that goes wrong when using a plugin --- make sure you trust the plugins you use!

All examples of plugin usage are available [here]().
